@@ -41,6 +41,28 @@ static int pax_seccomp_rules_add(scmp_filter_ctx ctx, int syscalls[], size_t num
 }
 #define pax_seccomp_rules_add(ctx, syscalls) pax_seccomp_rules_add(ctx, syscalls, ARRAY_SIZE(syscalls))
 
+static void
+pax_seccomp_sigal(__unused__ int signo, siginfo_t *info, __unused__ void *context)
+{
+	warn("seccomp violated: syscall %i", info->si_syscall);
+	fflush(stderr);
+#ifdef si_syscall
+	warn("  syscall = %s",
+		seccomp_syscall_resolve_num_arch(seccomp_arch_native(), info->si_syscall));
+#endif
+	kill(getpid(), SIGSYS);
+	_exit(1);
+}
+
+static void pax_seccomp_signal_init(void)
+{
+	struct sigaction act;
+	sigemptyset(&act.sa_mask);
+	act.sa_sigaction = pax_seccomp_sigal,
+	act.sa_flags = SA_SIGINFO | SA_RESETHAND;
+	sigaction(SIGSYS, &act, NULL);
+}
+
 static void pax_seccomp_init(bool allow_forking)
 {
 	/* Order determines priority (first == lowest prio).  */
@@ -50,13 +72,8 @@ static void pax_seccomp_init(bool allow_forking)
 
 		/* Then the stat family of functions.  */
 		SCMP_SYS(newfstatat),
-#ifdef __NR_fstat
 		SCMP_SYS(fstat),
-#endif
 		SCMP_SYS(fstat64),
-#ifdef __NR_fstatat
-		SCMP_SYS(fstatat),
-#endif
 		SCMP_SYS(fstatat64),
 		SCMP_SYS(lstat),
 		SCMP_SYS(lstat64),
@@ -68,9 +85,7 @@ static void pax_seccomp_init(bool allow_forking)
 
 		/* Then fd open family of functions.  */
 		SCMP_SYS(open),
-#ifdef __NR_openat
 		SCMP_SYS(openat),
-#endif
 
 		/* Then the memory mapping functions.  */
 		SCMP_SYS(mmap),
@@ -79,23 +94,14 @@ static void pax_seccomp_init(bool allow_forking)
 
 		/* Then the directory reading functions.  */
 		SCMP_SYS(getdents),
-#ifdef __NR_getdents64
 		SCMP_SYS(getdents64),
-#endif
 
 		/* Then the file reading functions.  */
-#ifdef __NR_pread
-		SCMP_SYS(pread),
-#endif
-#ifdef __NR_pread64
 		SCMP_SYS(pread64),
-#endif
 		SCMP_SYS(read),
 
 		/* Then the fd manipulation functions.  */
-#ifdef __NR_fcntl
 		SCMP_SYS(fcntl),
-#endif
 		SCMP_SYS(fcntl64),
 
 		/* After this point, just sort the list alphabetically.  */
@@ -110,14 +116,19 @@ static void pax_seccomp_init(bool allow_forking)
 		SCMP_SYS(getpid),
 		SCMP_SYS(gettid),
 		SCMP_SYS(ioctl),
-#ifdef __NR_lseek
 		SCMP_SYS(lseek),
-#endif
 		SCMP_SYS(_llseek),
 		SCMP_SYS(mprotect),
 
 		/* Syscalls listed because of sandbox.  */
 		SCMP_SYS(readlink),
+
+		/* Syscalls listed because of fakeroot.  */
+		SCMP_SYS(msgget),
+		SCMP_SYS(msgrcv),
+		SCMP_SYS(msgsnd),
+		SCMP_SYS(semget),
+		SCMP_SYS(semop),
 	};
 	int fork_syscalls[] = {
 		SCMP_SYS(clone),
@@ -131,7 +142,7 @@ static void pax_seccomp_init(bool allow_forking)
 		SCMP_SYS(waitid),
 		SCMP_SYS(waitpid),
 	};
-	scmp_filter_ctx ctx = seccomp_init(SCMP_ACT_TRAP);
+	scmp_filter_ctx ctx = seccomp_init(USE_DEBUG ? SCMP_ACT_TRAP : SCMP_ACT_KILL);
 	if (!ctx) {
 		warnp("seccomp_init failed");
 		return;
@@ -147,10 +158,16 @@ static void pax_seccomp_init(bool allow_forking)
 	/* We already called prctl. */
 	seccomp_attr_set(ctx, SCMP_FLTATR_CTL_NNP, 0);
 
+	if (USE_DEBUG)
+		pax_seccomp_signal_init();
+
 #ifndef __SANITIZE_ADDRESS__
 	/* ASAN does some weird stuff. */
-	if (seccomp_load(ctx) < 0)
-		warnp("seccomp_load failed");
+	if (seccomp_load(ctx) < 0) {
+		/* We have to assume that EINVAL == CONFIG_SECCOMP is disabled. */
+		if (errno != EINVAL)
+			warnp("seccomp_load failed");
+	}
 #endif
 
  done:
